@@ -1,5 +1,5 @@
 // build/run probes
-// Copyright (C) 2005-2016 Red Hat Inc.
+// Copyright (C) 2005-2018 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -233,6 +233,10 @@ compile_dyninst (systemtap_session& s)
   if (s.architecture == "i386")
     cmd.push_back("-march=i586");
 
+  // Need this for scripts where tautologies pass through to generated
+  // code.  XXX: but this is gcc-version-dependent
+  cmd.push_back("-Wno-tautological-compare");
+  
   for (size_t i = 0; i < s.c_macros.size(); ++i)
     cmd.push_back("-D" + s.c_macros[i]);
 
@@ -507,7 +511,8 @@ compile_pass (systemtap_session& s)
   o << "EXTRA_CFLAGS += $(call cc-option,-fno-ipa-icf)" << endl;
 
   // Assumes linux 2.6 kbuild
-  o << "EXTRA_CFLAGS += -Wno-unused " << WERROR << endl;
+  o << "EXTRA_CFLAGS += -Wno-unused $(call cc-option,-Wno-tautological-compare) " << WERROR
+    << endl;
   #if CHECK_POINTER_ARITH_PR5947
   o << "EXTRA_CFLAGS += -Wpointer-arith" << endl;
   #endif
@@ -931,7 +936,8 @@ make_tracequeries(systemtap_session& s, const map<string,string>& contents)
   omf << "EXTRA_CFLAGS := -g -Wno-implicit-function-declaration " << WERROR << endl;
   // RHBZ 655231: later rhel6 kernels' module-signing kbuild logic breaks out-of-tree modules
   omf << "CONFIG_MODULE_SIG := n" << endl;
-
+  // PR23488: need to override this kconfig, else we get no useful struct decls
+  omf << "CONFIG_DEBUG_INFO_REDUCED := " << endl;
   // PR18389: disable GCC's Identical Code Folding, since the stubs may look identical
   omf << "EXTRA_CFLAGS += $(call cc-option,-fno-ipa-icf)" << endl;
 
@@ -1009,6 +1015,9 @@ make_typequery_kmod(systemtap_session& s, const vector<string>& headers, string&
   // RHBZ 655231: later rhel6 kernels' module-signing kbuild logic breaks out-of-tree modules
   omf << "CONFIG_MODULE_SIG := n" << endl;
 
+  // PR23488: need to override this kconfig, else we get no useful struct decls
+  omf << "CONFIG_DEBUG_INFO_REDUCED := " << endl;
+  
   // NB: We use -include instead of #include because that gives us more power.
   // Using #include searches relative to the source's path, which in this case
   // is /tmp/..., so that's not helpful.  Using -include will search relative
