@@ -1,5 +1,5 @@
 // C++ interface to dwfl
-// Copyright (C) 2005-2015 Red Hat Inc.
+// Copyright (C) 2005-2018 Red Hat Inc.
 // Copyright (C) 2005-2007 Intel Corporation.
 // Copyright (C) 2008 James.Bottomley@HansenPartnership.com
 //
@@ -45,6 +45,7 @@ struct inline_instance_info;
 struct symbol_table;
 struct base_query;
 struct external_function_query;
+struct expression;
 
 enum lineno_t { ABSOLUTE, RELATIVE, WILDCARD, ENUMERATED };
 enum info_status { info_unknown, info_present, info_absent };
@@ -176,17 +177,8 @@ struct inline_instance_info : base_func_info
   bool operator<(const inline_instance_info& other) const;
 };
 
-/* We'll need some context when dwflpp::loc2c_error is called.
-   So we can attach some detailed information about (the location of)
-   the DWARF that failed to be translated. Needs updating before each
-   loc2c c_translate call. */
-struct loc2c_context
-{
-  Dwarf_Die *die;
-  Dwarf_Addr pc;
-public:
-  loc2c_context(): die(0), pc(0) {}
-};
+struct location;
+struct location_context;
 
 struct dwflpp
 {
@@ -206,10 +198,6 @@ struct dwflpp
   std::string module_name;
   std::string function_name;
 
-  // Some context for dwflpp::loc2c_error callback.
-  // Needs updating before each loc2c c_translate call.
-  struct loc2c_context l2c_ctx;
-
   dwflpp(systemtap_session & session, const std::string& user_module, bool kernel_p);
   dwflpp(systemtap_session & session, const std::vector<std::string>& user_modules, bool kernel_p);
   ~dwflpp();
@@ -417,6 +405,23 @@ struct dwflpp
                                  callers);
     }
 
+  template<typename T>
+  static int iterate_over_globals (Dwarf_Die *cu_die,
+                                   int (* callback)(Dwarf_Die*,
+                                                    bool,
+                                                    const std::string&,
+                                                    T*),
+                                   T *data)
+    {
+      // See comment block in iterate_over_modules()
+      return iterate_over_globals<void>(cu_die,
+                                        (int (*)(Dwarf_Die*,
+                                                 bool,
+                                                 const std::string&,
+                                                 void*))callback,
+                                        (void*)data);
+    }
+
   GElf_Shdr * get_section(std::string section_name, GElf_Shdr *shdr_mem,
                           Elf **elf_ret=NULL);
 
@@ -436,35 +441,39 @@ struct dwflpp
   bool inner_die_containing_pc(Dwarf_Die& scope, Dwarf_Addr addr,
                                Dwarf_Die& result);
 
-  std::string literal_stmt_for_local (std::vector<Dwarf_Die>& scopes,
-                                      Dwarf_Addr pc,
-                                      std::string const & local,
-                                      const target_symbol *e,
-                                      bool lvalue,
-                                      Dwarf_Die *die_mem);
+  bool literal_stmt_for_local (location_context &ctx,
+			       std::vector<Dwarf_Die>& scopes,
+			       std::string const & local,
+			       const target_symbol *e,
+			       bool lvalue,
+			       Dwarf_Die *die_mem);
   Dwarf_Die* type_die_for_local (std::vector<Dwarf_Die>& scopes,
-                                 Dwarf_Addr pc,
+				 Dwarf_Addr pc,
                                  std::string const & local,
                                  const target_symbol *e,
-                                 Dwarf_Die *die_mem);
+                                 Dwarf_Die *die_mem,
+				 bool lvalue);
 
-  std::string literal_stmt_for_return (Dwarf_Die *scope_die,
-                                       Dwarf_Addr pc,
-                                       const target_symbol *e,
-                                       bool lvalue,
-                                       Dwarf_Die *die_mem);
+  bool literal_stmt_for_return (location_context &ctx,
+				Dwarf_Die *scope_die,
+				const target_symbol *e,
+				bool lvalue,
+				Dwarf_Die *die_mem);
   Dwarf_Die* type_die_for_return (Dwarf_Die *scope_die,
                                   Dwarf_Addr pc,
                                   const target_symbol *e,
-                                  Dwarf_Die *die_mem);
+                                  Dwarf_Die *die_mem,
+				  bool lvalue);
 
-  std::string literal_stmt_for_pointer (Dwarf_Die *type_die,
-                                        const target_symbol *e,
-                                        bool lvalue,
-                                        Dwarf_Die *die_mem);
+  bool literal_stmt_for_pointer (location_context &ctx,
+				 Dwarf_Die *type_die,
+				 const target_symbol *e,
+				 bool lvalue,
+				 Dwarf_Die *die_mem);
   Dwarf_Die* type_die_for_pointer (Dwarf_Die *type_die,
                                    const target_symbol *e,
-                                   Dwarf_Die *die_mem);
+                                   Dwarf_Die *die_mem,
+				   bool lvalue);
 
   enum blacklisted_type
     {  blacklisted_none, // not blacklisted
@@ -537,23 +546,6 @@ private:
                                            const std::string& prefix, cu_type_cache_t *cache);
   static int global_alias_caching_callback_cus(Dwarf_Die *die, dwflpp *dw);
 
-  template<typename T>
-  static int iterate_over_globals (Dwarf_Die *cu_die,
-                                   int (* callback)(Dwarf_Die*,
-                                                    bool,
-                                                    const std::string&,
-                                                    T*),
-                                   T *data)
-    {
-      // See comment block in iterate_over_modules()
-      return iterate_over_globals<void>(cu_die,
-                                        (int (*)(Dwarf_Die*,
-                                                 bool,
-                                                 const std::string&,
-                                                 void*))callback,
-                                        (void*)data);
-    }
-
   template<typename T>
   static int iterate_over_types (Dwarf_Die *top_die,
                                  bool has_inner_types,
@@ -603,9 +595,7 @@ private:
 
   // This function generates code used for addressing computations of
   // target variables.
-  void emit_address (struct obstack *pool, Dwarf_Addr address);
-  static void loc2c_emit_address (void *arg, struct obstack *pool,
-                                  Dwarf_Addr address);
+  void emit_address (Dwarf_Addr address);
 
   void get_locals(std::vector<Dwarf_Die>& scopes, std::set<std::string>& locals);
   void get_locals_die(Dwarf_Die &die, std::set<std::string>& locals);
@@ -626,13 +616,13 @@ private:
   /* source file name, line and column info for pc in current cu. */
   const char *pc_line (Dwarf_Addr, int *, int *);
 
-  struct location *translate_location(struct obstack *pool,
-                                      Dwarf_Attribute *attr,
-                                      Dwarf_Die *die,
-                                      Dwarf_Addr pc,
-                                      Dwarf_Attribute *fb_attr,
-                                      struct location **tail,
-                                      const target_symbol *e);
+  location *translate_location(location_context *ctx,
+			       Dwarf_Attribute *attr,
+			       Dwarf_Die *die,
+			       Dwarf_Addr pc,
+			       Dwarf_Attribute *fb_attr,
+			       const target_symbol *e,
+			       location *input);
 
   bool find_struct_member(const target_symbol::component& c,
                           Dwarf_Die *parentdie,
@@ -640,28 +630,26 @@ private:
                           std::vector<Dwarf_Die>& dies,
                           std::vector<Dwarf_Attribute>& locs);
 
-  void translate_components(struct obstack *pool,
-                            struct location **tail,
-                            Dwarf_Addr pc,
+  void translate_components(location_context *ctx,
+			    Dwarf_Addr pc,
                             const target_symbol *e,
                             Dwarf_Die *vardie,
                             Dwarf_Die *typedie,
+			    bool lvalue,
                             unsigned first=0);
 
-  void translate_final_fetch_or_store (struct obstack *pool,
-                                       struct location **tail,
-                                       Dwarf_Addr module_bias,
-                                       Dwarf_Die *vardie,
-                                       Dwarf_Die *typedie,
-                                       bool lvalue,
-                                       const target_symbol *e,
-                                       std::string &,
-                                       std::string &,
-                                       Dwarf_Die *enddie);
-
-  std::string express_as_string (std::string prelude,
-                                 std::string postlude,
-                                 struct location *head);
+  void translate_base_ref (location_context &ctx, Dwarf_Word byte_size,
+			   bool signed_p, bool lvalue_p);
+  void translate_bitfield(location_context &ctx, Dwarf_Word byte_size,
+			  Dwarf_Word bit_offset, Dwarf_Word bit_size,
+			  bool signed_p);
+  void translate_final_fetch_or_store (location_context &ctx,
+				       Dwarf_Die *vardie,
+				       Dwarf_Die *typedie,
+				       bool lvalue,
+				       Dwarf_Die *enddie);
+  void translate_pointer(location_context &ctx, Dwarf_Die *typedie,
+			 bool lvalue);
 
   regex_t blacklist_func; // function/statement probes
   regex_t blacklist_func_ret; // only for .return probes
