@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2017 Red Hat Inc.
+// Copyright (C) 2005-2018 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -55,7 +55,7 @@ public:
   symresolution_info (systemtap_session& s, bool omniscient_unmangled = false);
 
   vardecl* find_var (interned_string name, int arity, const token *tok);
-  std::vector<functiondecl*> find_functions (const std::string& name, unsigned arity, const token *tok);
+  std::vector<functiondecl*> find_functions (functioncall *call, const std::string& name, unsigned arity, const token *tok);
   std::set<std::string> collect_functions(void);
 
   void visit_block (block *s);
