@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Map of addresses to disallow.
- * Copyright (C) 2005-2011 Red Hat Inc.
+ * Copyright (C) 2005-2016 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -16,6 +16,67 @@
  * certain addresses.
  */
 
+// PR12970 (populate runtime/bad-addr database) hasn't been
+// fixed. Until we're ready to populate the bad address database,
+// let's provide a "junior" version of lookup_bad_addr(), since that
+// will allow us to skip grabbing the lock and searching the empty
+// list.
+//
+// See <https://sourceware.org/bugzilla/show_bug.cgi?id=12970> for
+// more details.
+
+#include <linux/uaccess.h>
+
+#ifndef PR12970
+
+#if ! STP_PRIVILEGE_CONTAINS (STP_PRIVILEGE, STP_PR_STAPDEV) && \
+    ! STP_PRIVILEGE_CONTAINS (STP_PRIVILEGE, STP_PR_STAPSYS)
+#include <asm/processor.h> /* For TASK_SIZE */
+#endif
+
+static int
+lookup_bad_addr(const int type, const unsigned long addr, const size_t size)
+{
+  /* Is this a valid memory access?
+   * 
+   * Note we're using access_ok() here. This is only supposed to be
+   * called when we're in task context. Occasionally lookup_bad_addr()
+   * gets called when not in task context. If in_task() exists, only
+   * call access_ok() when we're in task context (otherwise we'll get
+   * a kernel warning). If we aren't in task context, we'll just do a
+   * range check.
+   */
+#if !defined(in_task)
+  if (size == 0 || ULONG_MAX - addr < size - 1
+      || !access_ok(type, (void *)addr, size))
+    return 1;
+#else
+  if (size == 0 || ULONG_MAX - addr < size - 1
+      || (in_task() && !access_ok(type, (void *)addr, size))
+      || (!in_task()
+#if defined(user_addr_max)
+	  && ((user_addr_max() - size) < addr)
+#endif
+	      ))
+    return 1;
+#endif
+
+#if ! STP_PRIVILEGE_CONTAINS (STP_PRIVILEGE, STP_PR_STAPDEV) && \
+    ! STP_PRIVILEGE_CONTAINS (STP_PRIVILEGE, STP_PR_STAPSYS)
+  /* Unprivileged users must not access memory while the context
+     does not refer to their own process.  */
+  if (! is_myproc ())
+    return 1;
+  /* Unprivileged users must not access kernel space memory.  */
+  if (addr + size > TASK_SIZE)
+    return 1;
+#endif
+
+  return 0;
+}
+
+#else /* PR12970 */
+
 struct addr_map_entry
 {
   unsigned long min;
@@ -106,14 +167,14 @@ lookup_addr_aux(unsigned long addr, size_t size, struct addr_map* map)
 #endif
 
 static int
-lookup_bad_addr(unsigned long addr, size_t size)
+lookup_bad_addr(const int type, const unsigned long addr, const size_t size)
 {
   struct addr_map_entry* result = 0;
   unsigned long flags;
 
-
   /* Is this a valid memory access?  */
-  if (size == 0 || ULONG_MAX - addr < size - 1)
+  if (size == 0 || ULONG_MAX - addr < size - 1
+      || !access_ok(type, (void *)addr, size))
     return 1;
 
 #if ! STP_PRIVILEGE_CONTAINS (STP_PRIVILEGE, STP_PR_STAPDEV) && \
@@ -222,4 +283,6 @@ delete_bad_addr_entry(struct addr_map_entry* entry)
 {
 }
 
+#endif /* PR12970 */
+
 #endif /* _STAPLINUX_ADDR_MAP_C_ */
