@@ -1,3 +1,14 @@
+/* -*- linux-c -*-
+ *
+ * /proc probe support functions
+ * Copyright (C) 2010-2016 Red Hat Inc.
+ *
+ * This file is part of systemtap, and is free software.  You can
+ * redistribute it and/or modify it under the terms of the GNU General
+ * Public License (GPL); either version 2, or (at your option) any
+ * later version.
+ */
+
 #ifndef _STP_PROCFS_PROBES_C_
 #define _STP_PROCFS_PROBES_C_
 
@@ -5,21 +16,14 @@
 #include <linux/fs.h>
 #include <linux/sched.h>
 
-#if 0
-// Currently we have to output _stp_procfs_data early in the
-// translation process.  It really should go here.
-struct _stp_procfs_data {
-	char *buffer;
-	size_t bufsize;
-	size_t count;
-};
-#endif
+#include "procfs-probes.h"
 
 struct stap_procfs_probe {
 	const char *path;
 	const struct stap_probe * const read_probe;
-	const struct stap_probe * const write_probe;
+        struct stap_probe **write_probes;
 
+        int num_write_probes;
 	char *buffer;
 	const size_t bufsize;
 	size_t count;
@@ -52,7 +56,7 @@ _stp_proc_open_file(struct inode *inode, struct file *filp)
 	struct stap_procfs_probe *spp;
 	int res;
 
-	spp = (struct stap_procfs_probe *)PDE(inode)->data;
+	spp = (struct stap_procfs_probe *)PDE_DATA(inode);
 	if (spp == NULL) {
 		return -EINVAL;
 	}
@@ -93,7 +97,8 @@ _stp_proc_open_file(struct inode *inode, struct file *filp)
 	if (likely(res == 0)) {
 		spp->opencount++;
 		filp->private_data = spp;
-		if ((filp->f_flags & O_ACCMODE) == O_RDONLY) {
+		if ((filp->f_flags & O_ACCMODE) == O_RDONLY
+		    && spp->read_probe != NULL) {
 			spp->buffer[0] = '\0';
 			spp->count = 0;
 			spp->needs_fill = 1;
@@ -129,9 +134,7 @@ _stp_proc_read_file(struct file *file, char __user *buf, size_t count,
 	struct stap_procfs_probe *spp = file->private_data;
 	ssize_t retval = 0;
 
-	/* If we don't have a probe read function, just return 0 to
-	 * indicate there isn't any data here. */
-	if (spp == NULL || spp->read_probe == NULL) {
+	if (spp == NULL || spp->buffer == NULL) {
 		goto out;
 	}
 
@@ -153,18 +156,22 @@ static ssize_t
 _stp_proc_write_file(struct file *file, const char __user *buf, size_t count,
 		     loff_t *ppos) 
 {
-	struct stap_procfs_probe *spp = file->private_data;
+        int i;
+        struct stap_procfs_probe *spp = file->private_data;
 	struct _stp_procfs_data pdata;
-	ssize_t len;
+	ssize_t len = -1;
 
-	/* If we don't have a write probe, return EIO. */
-	if (spp->write_probe == NULL) {
-		len = -EIO;
-		goto out;
+        /* If we don't have a write probe, return EIO. */
+        if (spp->num_write_probes == 0) {
+                len = -EIO;
+                goto out;
 	}
 
-	/* Handle the input buffer. */
-	len = _stp_process_write_buffer(spp, buf, count);
+	/* Handle the input buffer once for each write probe. */
+        for (i = 0; i < spp->num_write_probes; i++, spp->write_probes++)
+	        len = _stp_process_write_buffer(spp, buf, count);
+
+        spp->write_probes -= spp->num_write_probes;
 	if (len > 0) {
 		*ppos += len;
 	}
