@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Symbolic Lookup Functions
- * Copyright (C) 2005-2014 Red Hat Inc.
+ * Copyright (C) 2005-2018 Red Hat Inc.
  * Copyright (C) 2006 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -1045,7 +1045,7 @@ static void _stp_kmodule_update_address(const char* module,
       if (strcmp (_stp_modules[mi]->name, module))
         continue;
 
-      if (reloc && !strcmp (note_sectname, reloc)) {
+      if (!reloc || !strcmp (note_sectname, reloc)) {
         dbug_sym(1, "module %s special section %s address %#lx\n",
                  _stp_modules[mi]->name,
                  note_sectname,
