@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2016 Red Hat Inc.
+// Copyright (C) 2005-2018 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -80,9 +80,10 @@ struct compile_server_cache;
 // a vardecl instead of out here at the systemtap_session level.
 struct statistic_decl
 {
-  statistic_decl()
+  statistic_decl(int _stat_ops = 0)
     : type(none),
-      linear_low(0), linear_high(0), linear_step(0), bit_shift(0), stat_ops(0)
+      linear_low(0), linear_high(0), linear_step(0), bit_shift(0),
+      stat_ops(_stat_ops)
   {}
   enum { none, linear, logarithmic } type;
   int64_t linear_low;
@@ -144,7 +145,7 @@ public:
 
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
-  void setup_kernel_release (const char* kstr);
+  void setup_kernel_release (const std::string& kstr);
   void insert_loaded_modules ();
 
   // command line parsing
@@ -163,6 +164,7 @@ public:
   std::string script_file; // FILE
   std::string cmdline_script; // -e PROGRAM
   std::vector<std::string> additional_scripts; // -E SCRIPT
+  std::stringstream stdin_script;	       // stdin script (stap -)
   bool have_script;
   std::vector<std::string> include_path;
   int include_arg_start;
@@ -230,6 +232,7 @@ public:
   bool privilege_set;
   bool systemtap_v_check;
   bool tmpdir_opt_set;
+  bool read_stdin;
   bool monitor;
   int monitor_interval;
   int timeout; // in ms
@@ -251,12 +254,13 @@ public:
   bool suppress_time_limits;
   bool color_errors;
   bool interactive_mode;
+  bool run_example;
   bool pass_1a_complete;
 
   enum { color_never, color_auto, color_always } color_mode;
   enum { prologue_searching_never, prologue_searching_auto, prologue_searching_always } prologue_searching_mode;
 
-  enum { kernel_runtime, dyninst_runtime } runtime_mode;
+  enum { kernel_runtime, dyninst_runtime, bpf_runtime } runtime_mode;
   bool runtime_usermode_p() const { return runtime_mode == dyninst_runtime; }
 
   // NB: It is very important for all of the above (and below) fields
@@ -297,6 +301,9 @@ public:
   void set_try_server (int t = do_try_server);
   bool try_server () const { return try_server_status == do_try_server; }
 
+  // HTTP client/server
+  std::vector<std::string> http_servers;
+
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
 
@@ -340,6 +347,9 @@ public:
   std::vector<stapfile*> user_files;
   std::vector<stapfile*> library_files;
 
+  std::string script_name(); // usually user_files[0]->name
+  std::string script_basename(); // basename of script_name()
+
   // filters to run over all code before symbol resolution
   //   e.g. @cast expansion
   std::vector<update_visitor*> code_filters;
@@ -361,10 +371,10 @@ public:
 
   // resolved/compiled regular expressions for the run
   std::map<std::string, stapdfa*> dfas;
-  unsigned dfa_counter;  // used to give unique names
-  unsigned dfa_maxstate; // used for subexpression-tracking data structure
-  unsigned dfa_maxtag;   // ditto
-  bool need_tagged_dfa;  // triggered by /* pragma:tagged_dfa */
+  unsigned dfa_counter; // used to give unique names
+  unsigned dfa_maxmap;  // used for subexpression-tracking data structure
+  unsigned dfa_maxtag;  // ditto
+  bool need_tagged_dfa; // triggered by /* pragma:tagged_dfa */
 
   // Every probe in these groups must also appear in the
   // session.probes vector.
@@ -434,6 +444,8 @@ public:
 
   int target_namespaces_pid;
 
+  unsigned suppress_costly_diagnostics; /* set during processing of optional probes */
+
   const token* last_token;
   void print_token (std::ostream& o, const token* tok);
   void print_error (const semantic_error& e);
