@@ -7,7 +7,7 @@
  * Public License (GPL); either version 2, or (at your option) any
  * later version.
  *
- * Copyright (C) 2007-2015 Red Hat Inc.
+ * Copyright (C) 2007-2017 Red Hat Inc.
  */
 
 #include "staprun.h"
@@ -16,9 +16,17 @@
 #include <sys/utsname.h>
 #include <assert.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <dirent.h>
+#include <limits.h>
 #include "../git_version.h"
 #include "../version.h"
 
+#ifndef OPEN_MAX
+#define OPEN_MAX 256
+#endif
+
 /* variables needed by parse_args() */
 int verbose;
 int suppress_warnings;
@@ -39,9 +47,12 @@ off_t fsize_max;
 int fnum_max;
 int remote_id;
 const char *remote_uri;
+int read_stdin;
 int relay_basedir_fd;
 int color_errors;
 color_modes color_mode;
+int monitor;
+int monitor_interval;
 
 /* module variables */
 char *modname = NULL;
@@ -114,11 +125,14 @@ void parse_args(int argc, char **argv)
 	rename_mod = 0;
 	attach_mod = 0;
 	delete_mod = 0;
+        read_stdin = 0;
 	load_only = 0;
 	need_uprobes = 0;
 	daemon_mode = 0;
 	fsize_max = 0;
 	fnum_max = 0;
+	monitor = 0;
+        monitor_interval = 1;
         remote_id = -1;
         remote_uri = NULL;
         relay_basedir_fd = -1;
@@ -126,7 +140,7 @@ void parse_args(int argc, char **argv)
         color_errors = isatty(STDERR_FILENO)
                 && strcmp(getenv("TERM") ?: "notdumb", "dumb");
 
-	while ((c = getopt(argc, argv, "ALu::vhb:t:dc:o:x:N:S:DwRr:VT:C:"
+	while ((c = getopt(argc, argv, "ALu::vihb:t:dc:o:x:N:S:DwRr:VT:C:M:"
 #ifdef HAVE_OPENAT
                            "F:"
 #endif
@@ -172,6 +186,9 @@ void parse_args(int argc, char **argv)
 		case 'o':
 			outfile_name = optarg;
 			break;
+                case 'i':
+                        read_stdin = 1;
+                        break;
 		case 'R':
 			rename_mod = 1;
 			break;
@@ -215,8 +232,9 @@ void parse_args(int argc, char **argv)
                         }
 			break;
 		case 'V':
+                        /* PRERELEASE */
                         printf(_("Systemtap module loader/runner (version %s, %s)\n"
-                              "Copyright (C) 2005-2015 Red Hat, Inc. and others\n"
+                              "Copyright (C) 2005-2018 Red Hat, Inc. and others\n"
                               "This is free software; see the source for copying conditions.\n"),
                             VERSION, STAP_EXTENDED_VERSION);
                         _exit(0);
@@ -247,6 +265,13 @@ void parse_args(int argc, char **argv)
 				|| (color_mode == color_auto && isatty(STDERR_FILENO)
 						&& strcmp(getenv("TERM") ?: "notdumb", "dumb"));
 			break;
+		case 'M':
+			monitor = 1;
+			monitor_interval = atoi(optarg);
+			if (monitor_interval < 1) {
+				err(_("Invalid monitor interval\n"));
+			}
+			break;
 		default:
 			usage(argv[0],1);
 		}
@@ -259,7 +284,8 @@ void parse_args(int argc, char **argv)
 			err(_("File name is too long.\n"));
 			usage(argv[0],1);
 		}
-		ret = stap_strfloctime(tmp, PATH_MAX - 18, /* = _cpuNNN.SSSSSSSSSS */
+		ret = stap_strfloctime(tmp, PATH_MAX - 21,
+                                       /* = _cpuNNNNNN.SSSSSSSSSS */
 				       outfile_name, time(NULL));
 		if (ret < 0) {
 			err(_("Filename format is invalid or too long.\n"));
@@ -342,7 +368,10 @@ void usage(char *prog, int rc)
 	"-A              Attach to loaded systemtap module.\n"
 	"-C WHEN         Enable colored errors. WHEN must be either 'auto',\n"
 	"                'never', or 'always'. Set to 'auto' by default.\n"
-	"-d              Delete a module.  Only detached or unused modules\n"
+#ifdef HAVE_MONITOR_LIBS                 
+	"-M INTERVAL     Enable monitor mode.\n"
+#endif
+        "-d              Delete a module.  Only detached or unused modules\n"
 	"                the user has permission to access will be deleted. Use \"*\"\n"
 	"                (quoted) to delete all unused modules.\n"
         "-R              Have staprun create a new name for the module before\n"
@@ -549,6 +578,75 @@ err:
 	return -1;
 }
 
+/*
+ * In multithreaded programs, using fcntl() to set F_CLOEXEC
+ * (close-on-exec) on a file descriptor is subject to a race condition
+ * that another thread may call fork() and exec() between the time the
+ * file is opened and fcntl() is called.
+ *
+ * So, when possible (since Linux 2.6.23), we use the O_CLOEXEC flag
+ * when calling open() and openat() to avoid this race condition. When
+ * O_CLOEXEC isn't available, the best we can do is call fcntl()
+ * immediately after open() is called.
+ *
+ * The same logic applies to openat() and pipe().
+ */
+
+int open_cloexec(const char *pathname, int flags, mode_t mode)
+{
+#ifdef O_CLOEXEC
+	return open(pathname, flags | O_CLOEXEC, mode);
+#else
+	int fd = open(pathname, flags, mode);
+	if (fd >= 0) {
+		if (set_clexec(fd) < 0) {
+			close(fd);
+			fd = -1;
+		}
+	}
+	return fd;
+#endif
+}
+
+#ifdef HAVE_OPENAT
+int openat_cloexec(int dirfd, const char *pathname, int flags, mode_t mode)
+{
+#ifdef O_CLOEXEC
+	return openat(dirfd, pathname, flags | O_CLOEXEC, mode);
+#else
+	int fd = openat(dirfd, pathname, flags, mode);
+	if (fd >= 0) {
+		if (set_clexec(fd) < 0) {
+			close(fd);
+			fd = -1;
+		}
+	}
+	return fd;
+#endif
+}
+#endif
+
+int pipe_cloexec(int pipefd[2])
+{
+#ifdef O_CLOEXEC
+	return pipe2(pipefd, O_CLOEXEC);
+#else
+	int rc = pipe(pipefd);
+	if (rc == 0) {
+		if (set_clexec(pipefd[0]) < 0 || set_clexec(pipefd[1] < 0)) {
+			goto err;
+		}
+	}
+	return rc;
+
+err:
+	close(pipefd[0]);
+	close(pipefd[1]);
+	pipefd[0] = -1;
+	pipefd[1] = -1;
+	return -1;
+#endif
+}
 
 /**
  *      send_request - send request to kernel over control channel
@@ -658,3 +756,41 @@ char *parse_stap_color(const char *type)
 
 	return NULL; /* key not found */
 }
+
+void
+closefrom(int lowfd)
+{
+	long fd, maxfd;
+	char *endp;
+	struct dirent *dent;
+	DIR *dirp;
+
+	/* Check for a /proc/self/fd directory. */
+	if ((dirp = opendir("/proc/self/fd"))) {
+		int dir_fd = dirfd(dirp);
+		while ((dent = readdir(dirp)) != NULL) {
+			fd = strtol(dent->d_name, &endp, 10);
+			if (dent->d_name != endp && *endp == '\0'
+			    && fd >= 0 && fd < INT_MAX && fd >= lowfd
+			    && fd != dir_fd)
+				(void) close((int)fd);
+		}
+		(void) closedir(dirp);
+	}
+	else {
+		/*
+		 * Here we fall back on sysconf(). Why? It is possible
+		 * /proc isn't mounted, we're out of file descriptors,
+		 * etc., which could cause the opendir() to fail. Also
+		 * note thet it is possible to open a file descriptor
+		 * and then drop the rlimit such that it is below the
+		 * open fd.
+		 */
+		maxfd = sysconf(_SC_OPEN_MAX);
+		if (maxfd < 0)
+			maxfd = OPEN_MAX;
+
+		for (fd = lowfd; fd < maxfd; fd++)
+			(void) close((int) fd);
+	}
+}
