@@ -105,14 +105,12 @@ public:
 struct utrace_var_expanding_visitor: public var_expanding_visitor
 {
   utrace_var_expanding_visitor(systemtap_session& s, probe_point* l,
-			       const string& pn,
                                enum utrace_derived_probe_flags f):
-    sess (s), base_loc (l), probe_name (pn), flags (f),
+    var_expanding_visitor (s),
+    base_loc (l), flags (f),
     target_symbol_seen (false), add_block(NULL), add_probe(NULL) {}
 
-  systemtap_session& sess;
   probe_point* base_loc;
-  string probe_name;
   enum utrace_derived_probe_flags flags;
   bool target_symbol_seen;
   block *add_block;
@@ -139,8 +137,8 @@ utrace_derived_probe::utrace_derived_probe (systemtap_session &s,
     check_process_probe_kernel_support(s);
 
   // Expand local variables in the probe body
-  utrace_var_expanding_visitor v (s, l, name, flags);
-  v.replace (this->body);
+  utrace_var_expanding_visitor v (s, l, flags);
+  var_expand_const_fold_loop (s, this->body, v);
   target_symbol_seen = v.target_symbol_seen;
 
   // If during target-variable-expanding the probe, we added a new block
@@ -298,7 +296,7 @@ utrace_var_expanding_visitor::visit_target_symbol_cached (target_symbol* e)
       // calls at a time. The array will look like this:
       //
       //   _utrace_tvar_{name}_{num}
-      string aname = (string("_utrace_tvar_")
+      string aname = (string("__global_utrace_tvar_")
 		      + e->sym_name()
 		      + "_" + lex_cast(tick++));
       vardecl* vd = new vardecl;
@@ -506,7 +504,7 @@ utrace_var_expanding_visitor::visit_target_symbol_arg (target_symbol* e)
 	  functioncall* n = new functioncall; //same as the following
 	  n->tok = e->tok;
 	  n->function = "_utrace_syscall_arg";
-	  n->referent = 0;
+	  n->referents.clear();
 	  literal_number *num = new literal_number(i);
 	  num->tok = e->tok;
 	  n->args.push_back(num);
@@ -548,7 +546,7 @@ utrace_var_expanding_visitor::visit_target_symbol_arg (target_symbol* e)
         functioncall* n = new functioncall;
         n->tok = e->tok;
         n->function = "_utrace_syscall_arg";
-        n->referent = 0; // NB: must not resolve yet, to ensure inclusion in session
+        n->referents.clear(); // NB: must not resolve yet, to ensure inclusion in session
 
         literal_number *num = new literal_number(argnum - 1);
         num->tok = e->tok;
@@ -609,7 +607,7 @@ utrace_var_expanding_visitor::visit_target_symbol_context (target_symbol* e)
   functioncall* n = new functioncall;
   n->tok = e->tok;
   n->function = fname;
-  n->referent = 0; // NB: must not resolve yet, to ensure inclusion in session
+  n->referents.clear(); // NB: must not resolve yet, to ensure inclusion in session
 
   provide (n);
 }
@@ -704,6 +702,8 @@ struct utrace_builder: public derived_probe_builder
       }
     else if (has_path)
       {
+        if (path == "")
+            throw SEMANTIC_ERROR (_("empty module"));
         path = find_executable (path, sess.sysroot, sess.sysenv);
         sess.unwindsym_modules.insert (path);
         path_tgt = path_remove_sysroot(sess, path);
@@ -713,6 +713,8 @@ struct utrace_builder: public derived_probe_builder
 							has_path, path_tgt, pid,
 							flags));
   }
+
+  virtual string name() { return "utrace builder"; }
 };
 
 
