@@ -1,5 +1,5 @@
 // Copyright (C) Andrew Tridgell 2002 (original file)
-// Copyright (C) 2006-2014 Red Hat Inc. (systemtap changes)
+// Copyright (C) 2006-2018 Red Hat Inc. (systemtap changes)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -25,6 +25,10 @@
 #include <cassert>
 #include <ext/stdio_filebuf.h>
 #include <algorithm>
+#include <mutex>
+#include <functional> 
+#include <cctype>
+#include <locale>
 
 extern "C" {
 #include <elf.h>
@@ -40,9 +44,6 @@ extern "C" {
 #include <unistd.h>
 #include <regex.h>
 #include <stdarg.h>
-#ifndef SINGLE_THREADED
-#include <pthread.h>
-#endif
 }
 
 using namespace std;
@@ -159,7 +160,6 @@ copy_file(const string& src, const string& dest, bool verbose)
     }
 
   // Rename the temporary output file to the destination file.
-  unlink(dest.c_str());
   if (rename(tmp_name, dest.c_str()) == -1)
     {
       unlink(tmp_name);
@@ -230,6 +230,23 @@ remove_file_or_dir (const char *name)
   return 0;
 }
 
+
+int
+appendenv (const char *env_name, const string source)
+{
+  string dirname = source.substr(0, source.rfind("/"));
+  char *env = getenv(env_name);
+  string new_env;
+  
+  if (env)
+    new_env = string (env) + ":" + dirname;
+  else
+    new_env = dirname;
+    
+  return setenv(env_name, new_env.c_str(), 1);
+}
+
+
 /* Obtain the gid of the given group. */
 gid_t get_gid (const char *group_name)
 {
@@ -426,6 +443,66 @@ split_lines(const char *buf, size_t n)
   return lines;
 }
 
+static string
+follow_link(const string& name, const string& sysroot)
+{
+  char *linkname;
+  ssize_t r;
+  string retpath;
+  struct stat st;
+
+  const char *f = name.c_str();
+
+  lstat(f, &st);
+
+  linkname = (char *) malloc(st.st_size + 1);
+
+  if (linkname)
+    {
+      r = readlink(f, linkname, st.st_size + 1);
+      if (r != -1)
+	linkname[r] = '\0';
+      /*
+       * If we have non-empty sysroot and we got link that
+       * points to absolute path name, we need to look at
+       * this path relative to sysroot itself. access and
+       * stat will follow symbolic links correctly only in
+       * case with empty sysroot.
+       */
+      while (r != -1 && linkname && linkname[0] == '/')
+	{
+	  string fname1 = sysroot + linkname;
+	  const char *f1 = fname1.c_str();
+	  if (access(f1, X_OK) == 0
+	      && stat(f1, &st) == 0
+	      && S_ISREG(st.st_mode))
+	    {
+	      retpath = fname1;
+	      break;
+	    }
+	  else if (lstat(f1, &st) == 0
+		   && S_ISLNK(st.st_mode))
+	    {
+	      free(linkname);
+	      linkname = (char *) malloc(st.st_size + 1);
+	      if (linkname)
+		{
+		  r = readlink(f1, linkname, st.st_size + 1);
+		  if (r != -1)
+		    linkname[r] = '\0';
+		}
+	    }
+	  else
+	    {
+	      break;
+	    }
+	}
+    }
+  free(linkname);
+
+  return retpath;
+}
+
 // Resolve an executable name to a canonical full path name, with the
 // same policy as execvp().  A program name not containing a slash
 // will be searched along the $PATH.
@@ -450,6 +527,14 @@ string find_executable(const string& name, const string& sysroot,
   if (name.find('/') != string::npos) // slash in the path already?
     {
       retpath = sysroot + name;
+
+      const char *f = retpath.c_str();
+      if (sysroot != ""
+	  && lstat(f, &st) == 0
+	  && S_ISLNK(st.st_mode))
+	{
+	  retpath = follow_link(f, sysroot);
+	}
     }
   else // Nope, search $PATH.
     {
@@ -478,6 +563,16 @@ string find_executable(const string& name, const string& sysroot,
                   retpath = fname;
                   break;
                 }
+              else if (sysroot != ""
+                       && lstat(f, &st) == 0
+                       && S_ISLNK(st.st_mode))
+		{
+		  retpath = follow_link(f, sysroot);
+		  if (retpath != "")
+		    {
+		      break;
+		    }
+		}
             }
         }
     }
@@ -584,79 +679,61 @@ join(const vector<string>& vec, const string &delim)
 class spawned_pids_t {
   private:
     set<pid_t> pids;
+
 #ifndef SINGLE_THREADED
-    pthread_mutex_t mux_pids;
+    mutex mux_pids;
 #endif
 
+    unique_lock<mutex> lock()
+      {
+#ifndef SINGLE_THREADED
+        return unique_lock<mutex>(mux_pids);
+#else
+        return {};
+#endif
+      }
+
   public:
+
     bool contains (pid_t p)
       {
         stap_sigmasker masked;
+        auto guard = lock();
 
-#ifndef SINGLE_THREADED
-        pthread_mutex_lock(&mux_pids);
-#endif
         bool ret = (pids.count(p)==0) ? true : false;
-#ifndef SINGLE_THREADED
-        pthread_mutex_unlock(&mux_pids);
-#endif
 
         return ret;
       }
+
     bool insert (pid_t p)
       {
         stap_sigmasker masked;
+        auto guard = lock();
 
-#ifndef SINGLE_THREADED
-        pthread_mutex_lock(&mux_pids);
-#endif
         bool ret = (p > 0) ? pids.insert(p).second : false;
-#ifndef SINGLE_THREADED
-        pthread_mutex_unlock(&mux_pids);
-#endif
 
         return ret;
       }
+
     void erase (pid_t p)
       {
         stap_sigmasker masked;
+        auto guard = lock();
 
-#ifndef SINGLE_THREADED
-        pthread_mutex_lock(&mux_pids);
-#endif
         pids.erase(p);
-#ifndef SINGLE_THREADED
-        pthread_mutex_unlock(&mux_pids);
-#endif
       }
+
     int killall (int sig)
       {
         int ret = 0;
         stap_sigmasker masked;
+        auto guard = lock();
 
-#ifndef SINGLE_THREADED
-        pthread_mutex_lock(&mux_pids);
-#endif
         for (set<pid_t>::const_iterator it = pids.begin();
              it != pids.end(); ++it)
           ret = kill(*it, sig) ?: ret;
-#ifndef SINGLE_THREADED
-        pthread_mutex_unlock(&mux_pids);
-#endif
         return ret;
       }
-    spawned_pids_t()
-      {
-#ifndef SINGLE_THREADED
-        pthread_mutex_init(&mux_pids, NULL);
-#endif
-      }
-    ~spawned_pids_t()
-      {
-#ifndef SINGLE_THREADED
-        pthread_mutex_destroy (&mux_pids);
-#endif
-      }
 
 };
 static spawned_pids_t spawned_pids;
@@ -1070,6 +1147,26 @@ string unescape_glob_chars (const string& str)
   return op;
 }
 
+// PR23391, this is still incomplete but enough is
+// complete to handle "__{ia32,x64}_sys_$syscall"
+// functions.
+string csh_to_ksh (const string& csh)
+{
+  string ksh;
+  for (unsigned i=0; i<csh.size(); i++)
+    {
+      if (csh[i] == '{')
+        ksh += "@(";
+      else if (csh[i] == '}')
+        ksh += ')';
+      else if (csh[i] == ',')
+        ksh += '|';
+      else
+        ksh += csh[i];
+    }
+  return ksh;
+}
+
 bool identifier_string_needs_escape (const string& str)
 {
   for (unsigned i = 0; i < str.size (); i++)
@@ -1082,7 +1179,7 @@ bool identifier_string_needs_escape (const string& str)
   return false;
 }
 
-string escaped_indentifier_string (const string &str)
+string escaped_identifier_string (const string &str)
 {
   if (! identifier_string_needs_escape (str))
     return str;
@@ -1104,6 +1201,87 @@ string escaped_indentifier_string (const string &str)
   return op;
 }
 
+unsigned char
+octal_character (unsigned c)
+{
+  return '0' + c % 8;
+}
+
+string
+escaped_character (unsigned c)
+{
+  ostringstream o;
+  int oc = (int)c;
+
+  switch (oc)
+    {
+    case '\'':
+      o << "\\'";
+      break;
+
+    case '"':
+      o << "\\\"";
+      break;
+
+    case '\n':
+      o << "\\n";
+      break;
+
+    case '\t':
+      o << "\\t";
+      break;
+
+    case '\v':
+      o << "\\v";
+      break;
+
+    case '\b':
+      o << "\\b";
+      break;
+
+    case '\r':
+      o << "\\r";
+      break;
+
+    case '\f':
+      o << "\\f";
+      break;
+
+    case '\a':
+      o << "\\a";
+      break;
+
+    case '\\':
+      o << "\\\\";
+      break;
+
+    default:
+
+      if ((oc < 256) && isprint(oc))
+        {
+          o << (unsigned char) oc;
+        }
+      else
+        {
+          o << '\\' << octal_character(oc / 64)
+            << octal_character(oc / 8)
+            << octal_character(oc);
+        }
+    }
+  return o.str();
+}
+
+string
+escaped_literal_string (const string& str)
+{
+  string op;
+  for (unsigned i = 0; i < str.size (); i++)
+    {
+      op += escaped_character((unsigned)str[i]);
+    }
+  return op;
+}
+
 string
 normalize_machine(const string& machine)
 {
@@ -1387,4 +1565,227 @@ ppoll(struct pollfd *fds, nfds_t nfds,
 #endif
 
 
+int
+read_from_file (const string &fname, int &data)
+{
+  // C++ streams may not set errno in the even of a failure. However if we
+  // set it to 0 before each operation and it gets set during the operation,
+  // then we can use its value in order to determine what happened.
+  errno = 0;
+  ifstream f (fname.c_str ());
+  if (! f.good ())
+    {
+      clog << _F("Unable to open file '%s' for reading: ", fname.c_str());
+      goto error;
+    }
+
+  // Read the data;
+  errno = 0;
+  f >> data;
+  if (f.fail ())
+    {
+      clog << _F("Unable to read from file '%s': ", fname.c_str());
+      goto error;
+    }
+
+  // NB: not necessary to f.close ();
+  return 0; // Success
+
+ error:
+  if (errno)
+    clog << strerror (errno) << endl;
+  else
+    clog << _("unknown error") << endl;
+  return 1; // Failure
+}
+
+template <class T>
+int
+write_to_file (const string &fname, const T &data)
+{
+  // C++ streams may not set errno in the even of a failure. However if we
+  // set it to 0 before each operation and it gets set during the operation,
+  // then we can use its value in order to determine what happened.
+  errno = 0;
+  ofstream f (fname.c_str ());
+  if (! f.good ())
+    {
+      clog << _F("Unable to open file '%s' for writing: ", fname.c_str());
+      goto error;
+    }
+
+  // Write the data;
+  f << data;
+  errno = 0;
+  if (f.fail ())
+    {
+      clog << _F("Unable to write to file '%s': ", fname.c_str());
+      goto error;
+    }
+
+  // NB: not necessary to f.close ();
+  return 0; // Success
+
+ error:
+  if (errno)
+    clog << strerror (errno) << endl;
+  else
+    clog << _("unknown error") << endl;
+  return 1; // Failure
+}
+
+// Let's go ahead an instantiate a few variants of the write_to_file()
+// templated function.
+template int write_to_file (const string &fname, const string &data);
+template int write_to_file (const string &fname, const int &data);
+
+int
+flush_to_stream (const string &fname, ostream &o)
+{
+  // C++ streams may not set errno in the even of a failure. However if we
+  // set it to 0 before each operation and it gets set during the operation,
+  // then we can use its value in order to determine what happened.
+  errno = 0;
+  ifstream f (fname.c_str ());
+  if (! f.good ())
+    {
+      clog << _F("Unable to open file '%s' for reading: ", fname.c_str());
+      goto error;
+    }
+
+  // Stream the data
+
+  // NB: o << f.rdbuf() misbehaves for some reason, appearing to close o,
+  // which is unfortunate if o == clog or cout.
+  while (1)
+    {
+      errno = 0;
+      int c = f.get();
+      if (f.eof ()) return 0; // normal exit
+      if (! f.good()) break;
+      o.put(c);
+      if (! o.good()) break;
+    }
+
+  // NB: not necessary to f.close ();
+
+ error:
+  if (errno)
+    clog << strerror (errno) << endl;
+  else
+    clog << _("unknown error") << endl;
+  return 1; // Failure
+}
+
+// trim from start (in place)
+void
+ltrim(std::string &s)
+{
+  s.erase(s.begin(),
+	  std::find_if(s.begin(), s.end(),
+		       std::not1(std::ptr_fun<int, int>(std::isspace))));
+}
+
+// trim from end (in place)
+void
+rtrim(std::string &s)
+{
+  s.erase(std::find_if(s.rbegin(), s.rend(),
+	  std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
+}
+
+// trim from both ends (in place)
+void
+trim(std::string &s)
+{
+    ltrim(s);
+    rtrim(s);
+}
+
+// Tries to determine the name and version of the running Linux OS
+// distribution. Fills in the 'info' argument with (name, version)
+// strings. Returns true if it was able to retrieve the information.
+bool
+get_distro_info(vector<string> &info)
+{
+    string name, version;
+
+    // Getting the distro name and version is harder than it should
+    // be. We've got a multi-pronged strategy.
+    //
+    // (1) First, try the "lsb_release" executable, which may or may
+    // not exist on the system.
+    vector<string> cmd { "lsb_release", "--short", "--id" };
+    stringstream out;
+    int rc = stap_system_read(0, cmd, out);
+    if (rc == 0) {
+	name = out.str();
+
+	vector<string> cmd2 { "lsb_release", "--short", "--release" };
+	stringstream out2;
+	rc = stap_system_read(0, cmd2, out2);
+	if (rc == 0) {
+	    version = out2.str();
+	}
+    }
+
+    // (2) Look for the /etc/os-release file.
+    if (name.empty()) {
+	ifstream infile;
+	infile.open("/etc/os-release");
+	if (infile.is_open()) {
+	    string line;
+	    while (getline(infile, line)) {
+		vector<string> components;
+		tokenize(line, components, "=");
+		if (components.empty())
+		    continue;
+		transform(components[0].begin(), components[0].end(),
+			  components[0].begin(), ::tolower);
+		if (components[0] == "name") {
+		    string::size_type pos = components[1].find(' ');
+		    if (pos == string::npos)
+			name = components[1];
+		}
+		else if (components[0] == "version_id") {
+		    version = components[1];
+		}
+		else if (components[0] == "id" && name.empty()) {
+		    name = components[1];
+		}
+	    }
+	    infile.close();
+	}
+    }
+
+    // (3) Here we could look for /etc/*-release ('redhat', 'arch', 'gentoo',
+    // etc.) or /etc/*_version ('debian', etc.), if needed.
+
+    info.clear();
+    if (! name.empty()) {
+	trim(name);
+	// If the string is quoted, remove the quotes.
+	if (*name.begin() == '"') {
+	    name.erase(0, 1);
+	    name.erase(name.size() - 1);
+	    trim(name);
+	}
+    }
+    if (! version.empty()) {
+	trim(version);
+	// If the string is quoted, remove the quotes.
+	if (*version.begin() == '"') {
+	    version.erase(0, 1);
+	    version.erase(version.size() - 1);
+	    trim(version);
+	}
+    }
+    if (! name.empty()) {
+	info.push_back(name);
+	info.push_back(version);
+	return true;
+    }
+    return false;
+}
+
 /* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
