@@ -77,6 +77,7 @@ bool copy_file(const std::string& src, const std::string& dest,
 int create_dir(const char *dir, int mode = 0777);
 int remove_file_or_dir(const char *dir);
 extern "C" gid_t get_gid (const char *group_name);
+int appendenv (const char *env_name, const std::string source);
 bool in_group_id (gid_t target_gid);
 std::string getmemusage ();
 void tokenize(const std::string& str, std::vector<std::string>& tokens,
@@ -119,8 +120,11 @@ int regexp_match (const std::string& value, const std::string& re, std::vector<s
 bool contains_glob_chars (const std::string &str);
 std::string escape_glob_chars (const std::string& str);
 std::string unescape_glob_chars (const std::string& str);
+std::string csh_to_ksh (const std::string& csh);
 bool identifier_string_needs_escape (const std::string& str);
-std::string escaped_indentifier_string (const std::string& str);
+std::string escaped_identifier_string (const std::string& str);
+std::string escaped_character (unsigned c);
+std::string escaped_literal_string (const std::string& str);
 std::string kernel_release_from_build_tree (const std::string &kernel_build_tree, int verbose = 0);
 std::string normalize_machine(const std::string& machine);
 int elf_class_from_normalized_machine(const std::string& machine);
@@ -407,6 +411,16 @@ int ppoll(struct pollfd *fds, nfds_t nfds,
 #endif
 
 
+int read_from_file (const std::string &fname, int &data);
+template <class T>
+int write_to_file (const std::string &fname, const T &data);
+int flush_to_stream (const std::string &fname, std::ostream &o);
+
+void ltrim(std::string &s);
+void rtrim(std::string &s);
+void trim(std::string &s);
+
+bool get_distro_info(std::vector<std::string> &info);
 #endif // UTIL_H
 
 /* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
